"
 COPYRIGHT (c) 1991 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

"{ Package: 'stx:libtool' }"

ApplicationModel subclass:#NewFileBrowser
	instanceVariableNames:''
	classVariableNames:'DefaultIcon Icons DefaultCommandPerMIME'
	poolDictionaries:''
	category:'Interface-Browsers'
!

!NewFileBrowser class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1991 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    NewFileBrowser is based on Filebrowser created by Claus Gittinger

    WARNING: files edited with FileBrowser will have leading spaces (multiple-8)
             being replaced by tabs. If tabs are to be preserved at other
             positions (for example, sendmail-config files) they will be
             corrupt after being written.

    [instance variables]:

        checkDelta      <Integer>       number of seconds of check interval
                                        (looks ever so often if shown directory
                                         has changed). You may make this number
                                        higher, if your network-times are
                                        incorrect and thus, the filebrowser
                                        checks too often.

        compressTabs    <Boolean>       if true, leading spaces will be
                                        replaced by tabs when saving text

    some of the defaults (long/short list etc.) can be set by the resource file;
    see FileBrowser>>initialize for more details..

    [author:]
        Christian Penk

    [start with:]
        NewFileBrowser open
"
! !

!NewFileBrowser class methodsFor:'instance creation'!

on:aDirectoryPath
    "return a new NewFileBrowser in a pathname"

    ^ (self new currentDirectory:aDirectoryPath)

    "
     (NewFileBrowser on:'/usr/local/bin') open
     (NewFileBrowser on:'/etc'          ) open
     (NewFileBrowser on:'..'            ) open
     (NewFileBrowser on:'.'             ) open
    "
!

openOn:aDirectoryPath withExtent:extent
    "start a new NewFileBrowser in a pathname"

    ^ (self on:aDirectoryPath) openWithExtent:extent

    "
     NewFileBrowser openOn:'/etc'           withExtent:200@300
     NewFileBrowser openOn:'..'             withExtent:200@300
     NewFileBrowser openOn:'.'              withExtent:200@300
    "
!

openOnFileNamed:aFilename
    "start a new NewFileBrowser on a file"

    |f browser|

    f := aFilename asFilename.
    f isDirectory ifTrue:[
        ^ self on:aFilename
    ].

    browser := self on:f directoryName.
    browser updateCurrentDirectory.
    browser showFile:f baseName.

    ^ browser open.

    "
     NewFileBrowser openOnFileNamed:'Makefile'
     NewFileBrowser openOnFileNamed:'../Makefile'
     NewFileBrowser openOnFileNamed:'/tmp/foo'
    "

    "Modified: / 17.6.1998 / 11:25:29 / cg"
! !

!NewFileBrowser class methodsFor:'class initialization'!

icons
    Icons isNil ifTrue:[
        self initializeIcons
    ].
    ^ Icons
!

initializeDefaultCommands
    DefaultCommandPerMIME := Dictionary new.

    DefaultCommandPerMIME at:'application/x-tar-compressed' put:'gunzip < %1 | tar tvf -'.
    DefaultCommandPerMIME at:'application/pdf'              put:'acroread %1'.

    "
     self initializeDefaultCommands
    "

    "Modified: / 1.8.1998 / 17:39:27 / cg"

!

initializeIcons
    |resources|

    resources := self classResources.

    Icons := Dictionary new.

    #(
        "/ internal-type to icon mappings.
        (#directory       'ICON_DIRECTORY'        'tiny_yellow_dir.xpm'       )
        (#directoryLocked 'ICON_DIRECTORY_LOCKED' 'tiny_yellow_dir_locked.xpm')
        (#directoryLink   'ICON_DIRECTORY_LINK'   'tiny_yellow_dir_link.xpm'  )
        (#file            'ICON_FILE'             'tiny_file_plain.xpm'       )
        (#fileLink        'ICON_FILE_LINK'        'tiny_file_link.xpm'        )
        (#fileLocked      'ICON_FILE_LOCKED'      'tiny_file_lock.xpm'        )
        (#imageFile       'ICON_IMAGE_FILE'       'tiny_file_pix.xpm'         )
        (#textFile        'ICON_TEXT_FILE'        'tiny_file_text.xpm'        )
        (#executableFile  'ICON_EXECUTABLEFILE'   'tiny_file_exec.xpm'        )

        "/ mime-type to icon mappings.
        ('text/plain'                     nil     'tiny_file_text.xpm'        )
        ('text/html'                      nil     'tiny_file_text.xpm'        )
        ('application/postscript'         nil     'tiny_file_st.xpm'          )
        ('application/x-smalltalk-source' nil     'tiny_file_st.xpm'          )
        ('application/x-java-source'      nil     'tiny_file_java.xpm'        )
        ('image'                          nil     'tiny_file_pix.xpm'         )
        ('audio'                          nil     'tiny_file_sound.xpm'       )
     ) do:[:entry |
        |key resource defaultName nm img|

        key := entry at:1.
        resource := entry at:2.
        defaultName := entry at:3.

        resource notNil ifTrue:[
            nm := resources at:resource default:nil.
        ].
        nm isNil ifTrue:[
            nm := 'xpmBitmaps/document_images/' , defaultName
        ].
        Icons at:key put:(Smalltalk imageFromFileNamed:nm forClass:self).
    ]

    "
     self initializeIcons
    "

    "Modified: / 1.8.1998 / 17:39:27 / cg"
! !

!NewFileBrowser class methodsFor:'defaults'!

defaultIcon
    "return the file browsers default window icon"

    <resource: #style (#FILEBROWSER_ICON #FILEBROWSER_ICON_FILE)>

    |nm i res|

    (i := DefaultIcon) isNil ifTrue:[
        res := self classResources.
        i := res at:'FILEBROWSER_ICON' default:nil.
        i isNil ifTrue:[
            nm := res at:'FILEBROWSER_ICON_FILE' default:'FBrowser.xbm'.
            i := Smalltalk imageFromFileNamed:nm forClass:self.
            i isNil ifTrue:[
                i := StandardSystemView defaultIcon
            ]
        ].
        i notNil ifTrue:[
            DefaultIcon := i := i onDevice:Display
        ]
    ].
    ^ i

    "Modified: 19.3.1997 / 20:48:34 / ca"
    "Modified: 15.8.1997 / 15:27:19 / cg"
! !

!NewFileBrowser class methodsFor:'interface specs'!

fileSearchDialogSpec
    "This resource specification was automatically generated
     by the UIPainter of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the UIPainter may not be able to read the specification."

    "
     UIPainter new openOnClass:FileBrowser andSelector:#fileSearchDialogSpec
    "

    <resource: #canvas>

    ^
     
       #(#FullSpec
          #window: 
           #(#WindowSpec
              #name: 'File Search'
              #layout: #(#LayoutFrame 4 0 50 0 315 0 349 0)
              #level: 0
              #label: 'File Search'
              #min: #(#Point 10 10)
              #max: #(#Point 1280 1024)
              #bounds: #(#Rectangle 4 50 316 350)
              #usePreferredExtent: false
              #returnIsOKInDialog: true
              #escapeIsCancelInDialog: true
          )
          #component: 
           #(#SpecCollection
              #collection: 
               #(
                 #(#LabelSpec
                    #name: 'Label1'
                    #layout: #(#LayoutFrame 0 0 10 0 0 1 32 0)
                    #label: 'Search for files named:'
                    #translateLabel: true
                    #adjust: #left
                )
                 #(#InputFieldSpec
                    #name: 'EntryField1'
                    #layout: #(#LayoutFrame 10 0 36 0 305 0 58 0)
                    #tabable: true
                    #model: #namePatternHolder
                )
                 #(#CheckBoxSpec
                    #name: 'CheckBox1'
                    #layout: #(#LayoutFrame 7 0 66 0 143 0 88 0)
                    #tabable: true
                    #model: #ignoreCaseInName
                    #label: 'Ignore case'
                    #translateLabel: true
                )
                 #(#LabelSpec
                    #name: 'Label2'
                    #layout: #(#LayoutFrame 0 0.0 107 0 0 1.0 129 0)
                    #label: 'Containing the string:'
                    #translateLabel: true
                    #adjust: #left
                )
                 #(#InputFieldSpec
                    #name: 'EntryField2'
                    #layout: #(#LayoutFrame 10 0 133 0 305 0 155 0)
                    #enableChannel: #notSearchForSameContents
                    #tabable: true
                    #model: #contentsPatternHolder
                )
                 #(#CheckBoxSpec
                    #name: 'CheckBox2'
                    #layout: #(#LayoutFrame 6 0 163 0 142 0 185 0)
                    #enableChannel: #notSearchForSameContents
                    #tabable: true
                    #model: #ignoreCaseInContents
                    #label: 'Ignore case'
                    #translateLabel: true
                )
                 #(#LabelSpec
                    #name: 'Label3'
                    #layout: #(#LayoutFrame 0 0.0 223 0 -30 1.0 245 0)
                    #label: 'Containing same contents as selected:'
                    #translateLabel: true
                    #adjust: #left
                )
                 #(#DividerSpec
                    #name: 'Separator1'
                    #layout: #(#LayoutFrame 0 0.0 97 0 0 1.0 101 0)
                )
                 #(#CheckToggleSpec
                    #name: 'CheckToggle1'
                    #layout: #(#LayoutOrigin -25 1 225 0)
                    #tabable: true
                    #model: #searchForSameContents
                    #enableChannel: #searchForSameContentsEnabled
                    #isTriggerOnDown: true
                    #showLamp: false
                    #lampColor: #(#Color 100.0 100.0 0.0)
                )
                 #(#HorizontalPanelViewSpec
                    #name: 'HorizontalPanel1'
                    #layout: #(#LayoutFrame 0 0.0 -30 1 0 1.0 0 1.0)
                    #component: 
                     #(#SpecCollection
                        #collection: 
                         #(
                           #(#ActionButtonSpec
                              #name: 'Button1'
                              #label: 'Cancel'
                              #translateLabel: true
                              #tabable: true
                              #model: #cancel
                              #extent: #(#Point 151 25)
                          )
                           #(#ActionButtonSpec
                              #name: 'Button2'
                              #label: 'Search'
                              #translateLabel: true
                              #tabable: true
                              #model: #accept
                              #isDefault: true
                              #extent: #(#Point 152 25)
                          )
                        )
                    )
                    #horizontalLayout: #fitSpace
                    #verticalLayout: #centerMax
                    #horizontalSpace: 3
                    #verticalSpace: 3
                )
              )
          )
      )
!

windowSpec
    "This resource specification was automatically generated
     by the UIPainter of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the UIPainter may not be able to read the specification."

    "
     UIPainter new openOnClass:NewFileBrowser andSelector:#windowSpec
     NewFileBrowser new openInterface:#windowSpec
     NewFileBrowser open
    "

    <resource: #canvas>

    ^ 
     #(#FullSpec
        #name: #windowSpec
        #window: 
       #(#WindowSpec
          #label: #NewFileBrowser
          #name: #NewFileBrowser
          #min: #(#Point 10 10)
          #max: #(#Point 1152 864)
          #bounds: #(#Rectangle 41 73 807 673)
          #menu: #menuSpec
        )
        #component: 
       #(#SpecCollection
          #collection: #(
           #(#VariableHorizontalPanelSpec
              #name: 'DirectoryHorizontalPanel'
              #layout: #(#LayoutFrame 0 0.0 0 0 0 1.0 20 0)
              #component: 
             #(#SpecCollection
                #collection: #(
                 #(#FilenameInputFieldSpec
                    #name: 'FilenameEntryField1'
                    #style: #(#FontDescription #helvetica #bold #roman 12)
                    #acceptOnPointerLeave: false
                    #postBuildCallback: #postBuildPathView:
                  )
                 #(#InputFieldSpec
                    #name: 'FilterEnntry'
                    #acceptOnReturn: true
                    #acceptOnTab: true
                    #acceptOnLostFocus: true
                    #acceptOnPointerLeave: false
                  )
                 )
               
              )
              #handles: #(#Any 0.82376 1.0)
            )
           #(#VariableVerticalPanelSpec
              #name: 'VariableVerticalPanel1'
              #layout: #(#LayoutFrame 0 0.0 20 0 0 1.0 -20 1)
              #component: 
             #(#SpecCollection
                #collection: #(
                 #(#DataSetSpec
                    #name: 'FileTable'
                    #hasHorizontalScrollBar: true
                    #hasVerticalScrollBar: true
                    #has3Dsepartors: false
                  )
                 #(#TextEditorSpec
                    #name: 'TextEditor1'
                    #hasHorizontalScrollBar: true
                    #hasVerticalScrollBar: true
                  )
                 )
               
              )
              #handles: #(#Any 0.5 1.0)
            )
           #(#ViewSpec
              #name: 'CommandBox'
              #layout: #(#LayoutFrame 0 0.0 -20 1 0 1.0 0 1.0)
              #component: 
             #(#SpecCollection
                #collection: #(
                 #(#InputFieldSpec
                    #name: 'CommandField'
                    #layout: #(#LayoutFrame 0 0.0 0 0.0 0 1.0 0 1.0)
                    #acceptOnReturn: true
                    #acceptOnTab: true
                    #acceptOnLostFocus: true
                    #acceptOnPointerLeave: false
                  )
                 #(#ActionButtonSpec
                    #label: 'kill'
                    #name: 'kill'
                    #layout: #(#LayoutFrame 0 0 0 0 50 0 20 0)
                    #translateLabel: true
                  )
                 #(#ActionButtonSpec
                    #label: 'pause'
                    #name: 'Pause'
                    #layout: #(#LayoutFrame 50 0 0 0 100 0 20 0)
                    #translateLabel: true
                  )
                 )
               
              )
            )
           )
         
        )
      )
! !

!NewFileBrowser class methodsFor:'menu specs'!

menuSpec
    "This resource specification was automatically generated
     by the MenuEditor of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the MenuEditor may not be able to read the specification."

    "
     MenuEditor new openOnClass:FileBrowser andSelector:#menuSpec
     (Menu new fromLiteralArrayEncoding:(FileBrowser menuSpec)) startUp
    "

    <resource: #menu>

    ^ 
     #(#Menu
        #(
         #(#MenuItem
            #label: 'File'
            #translateLabel: true
            #submenu: 
           #(#Menu
              #(
               #(#MenuItem
                  #label: 'Spawn'
                  #translateLabel: true
                  #value: #fileSpawn
                )
               #(#MenuItem
                  #label: '-'
                )
               #(#MenuItem
                  #label: 'Open'
                  #translateLabel: true
                  #isVisible: false
                  #value: #menuOpen
                  #enabled: #hasSelection
                )
               #(#MenuItem
                  #label: 'Open selected Filename'
                  #translateLabel: true
                  #isVisible: #hasFilenameSelectionInCodeView
                  #value: #openSelectedFilename
                  #enabled: #hasFilenameSelectionInCodeView
                )
               #(#MenuItem
                  #label: 'FileIn'
                  #translateLabel: true
                  #value: #fileFileIn
                  #enabled: #hasSelection
                )
               #(#MenuItem
                  #label: 'FileInTo NameSpace...'
                  #translateLabel: true
                  #value: #fileFileInToNameSpace
                  #enabled: #hasSelection
                )
               #(#MenuItem
                  #label: '-'
                )
               #(#MenuItem
                  #label: 'New'
                  #translateLabel: true
                  #submenu: 
                 #(#Menu
                    #(
                     #(#MenuItem
                        #label: 'Directory...'
                        #translateLabel: true
                        #value: #newDirectory
                      )
                     #(#MenuItem
                        #label: 'File...'
                        #translateLabel: true
                        #value: #newFile
                      )
                     #(#MenuItem
                        #label: 'Hard Link...'
                        #translateLabel: true
                        #isVisible: #systemIsUnix
                        #value: #newHardLink
                      )
                     #(#MenuItem
                        #label: 'Symbolic Link...'
                        #translateLabel: true
                        #isVisible: #systemIsUnix
                        #value: #newSoftLink
                      )
                     )
                    nil
                    nil
                  )
                )
               #(#MenuItem
                  #label: 'Remove'
                  #translateLabel: true
                  #value: #fileRemove
                  #enabled: #hasSelection
                )
               #(#MenuItem
                  #label: 'Rename'
                  #translateLabel: true
                  #value: #fileRename
                  #enabled: #hasSelection
                )
               #(#MenuItem
                  #label: '-'
                )
               #(#MenuItem
                  #label: 'Properties...'
                  #translateLabel: true
                  #value: #fileGetLongInfo
                  #enabled: #hasSelection
                )
               #(#MenuItem
                  #label: '-'
                  #isVisible: #javaSupportLoaded
                )
               #(#MenuItem
                  #label: 'Add to Java ClassPath'
                  #translateLabel: true
                  #isVisible: #javaSupportLoaded
                  #value: #fileAddToJavaClassPath
                  #enabled: #canAddToClassPath
                )
               #(#MenuItem
                  #label: 'Remove from Java ClassPath'
                  #translateLabel: true
                  #isVisible: #javaSupportLoaded
                  #value: #fileRemoveFromJavaClassPath
                  #enabled: #canRemoveFromClassPath
                )
               #(#MenuItem
                  #label: 'Add to Java SourcePath'
                  #translateLabel: true
                  #isVisible: #javaSupportLoaded
                  #value: #fileAddToJavaSourcePath
                  #enabled: #canAddToSourcePath
                )
               #(#MenuItem
                  #label: 'Remove from Java SourcePath'
                  #translateLabel: true
                  #isVisible: #javaSupportLoaded
                  #value: #fileRemoveFromJavaSourcePath
                  #enabled: #canRemoveFromSourcePath
                )
               #(#MenuItem
                  #label: '-'
                )
               #(#MenuItem
                  #label: 'Exit'
                  #translateLabel: true
                  #value: #menuExit
                )
               )
              nil
              nil
            )
          )
         #(#MenuItem
            #label: 'Directory'
            #translateLabel: true
            #submenuChannel: #directoryMenuSpec
          )
         #(#MenuItem
            #label: 'Edit'
            #translateLabel: true
            #submenu: 
           #(#Menu
              #(
               #(#MenuItem
                  #label: 'Get Contents'
                  #translateLabel: true
                  #value: #fileGet
                  #enabled: #hasSelection
                )
               #(#MenuItem
                  #label: 'Insert Contents'
                  #translateLabel: true
                  #value: #fileInsert
                  #enabled: #hasSelection
                )
               #(#MenuItem
                  #label: '-'
                )
               #(#MenuItem
                  #label: 'Copy File List'
                  #translateLabel: true
                  #value: #copyFileList
                )
               #(#MenuItem
                  #label: 'Copy Selected Filename'
                  #translateLabel: true
                  #value: #copySelectedFileName
                  #enabled: #hasSelection
                )
               #(#MenuItem
                  #label: 'Copy Selected Pathname'
                  #translateLabel: true
                  #value: #copySelectedPathName
                  #enabled: #hasSelection
                )
               #(#MenuItem
                  #label: 'Copy Command History'
                  #translateLabel: true
                  #value: #copyCommandHistory
                )
               #(#MenuItem
                  #label: '-'
                )
               #(#MenuItem
                  #label: 'Visited Files'
                  #translateLabel: true
                  #submenuChannel: #visitedFileMenuSpec
                )
               )
              nil
              nil
            )
          )
         #(#MenuItem
            #label: 'View'
            #translateLabel: true
            #submenu: 
           #(#Menu
              #(
               #(#MenuItem
                  #label: 'Show Details'
                  #translateLabel: true
                  #indication: #showingDetails
                )
               #(#MenuItem
                  #label: 'Show Time && Date'
                  #translateLabel: true
                  #indication: #showingTimeAndDate
                  #enabled: #showingDetails
                )
               #(#MenuItem
                  #label: 'Show Hidden Files'
                  #translateLabel: true
                  #indication: #showingHiddenFiles
                )
               #(#MenuItem
                  #label: 'Show Big Image Preview'
                  #translateLabel: true
                  #indication: #showingBigImagePreview
                )
               #(#MenuItem
                  #label: '-'
                )
               #(#MenuItem
                  #label: 'Sort by Name'
                  #translateLabel: true
                  #choice: #sortByWhat
                  #choiceValue: #name
                )
               #(#MenuItem
                  #label: 'Sort by Type'
                  #translateLabel: true
                  #choice: #sortByWhat
                  #choiceValue: #type
                )
               #(#MenuItem
                  #label: 'Sort by Time'
                  #translateLabel: true
                  #choice: #sortByWhat
                  #choiceValue: #time
                )
               #(#MenuItem
                  #label: 'Ignore Case in Sort'
                  #translateLabel: true
                  #indication: #sortCaseless
                )
               #(#MenuItem
                  #label: '-'
                )
               #(#MenuItem
                  #label: 'Encoding...'
                  #translateLabel: true
                  #value: #fileEncoding
                )
               #(#MenuItem
                  #label: 'DOS EndOfLine Mode'
                  #translateLabel: true
                  #indication: #dosEOLMode
                )
               #(#MenuItem
                  #label: '-'
                )
               #(#MenuItem
                  #label: 'Update'
                  #translateLabel: true
                  #value: #updateCurrentDirectory
                )
               #(#MenuItem
                  #label: 'AutoUpdate'
                  #translateLabel: true
                  #indication: #autoUpdate
                )
               )
              nil
              nil
            )
          )
         #(#MenuItem
            #label: 'Tools'
            #translateLabel: true
            #submenuChannel: #toolsMenuSpec
          )
         #(#MenuItem
            #label: 'CVS'
            #translateLabel: true
            #submenu: 
           #(#Menu
              #(
               #(#MenuItem
                  #label: 'Update selected files/directories'
                  #translateLabel: true
                  #value: #cvsUpdateSelection
                  #enabled: #hasSelection
                )
               #(#MenuItem
                  #label: 'Update directory local'
                  #translateLabel: true
                  #value: #cvsUpdateDirectoryLocal
                )
               #(#MenuItem
                  #label: 'Update directory recursive'
                  #translateLabel: true
                  #value: #cvsUpdateDirectoryRecursive
                )
               #(#MenuItem
                  #label: '-'
                )
               #(#MenuItem
                  #label: 'Commit...'
                  #translateLabel: true
                  #value: #cvsCommitSelection
                )
               #(#MenuItem
                  #label: 'Add && Commit...'
                  #translateLabel: true
                  #value: #cvsAddAndCommitSelection
                )
               #(#MenuItem
                  #label: '-'
                )
               #(#MenuItem
                  #label: 'Remove file && CVS Container...'
                  #translateLabel: true
                  #value: #cvsRemoveFileAndContainer
                  #enabled: #canRemoveCVSContainer
                )
               )
              nil
              nil
            )
          )
         #(#MenuItem
            #label: 'Help'
            #translateLabel: true
            #startGroup: #right
            #submenu: 
           #(#Menu
              #(
               #(#MenuItem
                  #label: 'FileBrowser Documentation'
                  #translateLabel: true
                  #value: #openHTMLDocument:
                  #argument: 'tools/fbrowser/TOP.html'
                )
               #(#MenuItem
                  #label: '-'
                )
               #(#MenuItem
                  #label: 'About FileBrowser...'
                  #translateLabel: true
                  #value: #openAboutThisApplication
                )
               )
              nil
              nil
            )
          )
         )
        nil
        nil
      )
! !

!NewFileBrowser class methodsFor:'queries'!

isVisualStartable
    "return true, if this application can be started via #open.
     (to allow start of a change browser via double-click in the browser)"

    ^ true
! !

!NewFileBrowser methodsFor:'startup & release'!

postBuildPathView:aWidget
    "
        set the pramters that not can be changed in the window spec
    "

    aWidget level:-1.
    aWidget backgroundColor:(aWidget superView viewBackground).
! !

!NewFileBrowser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libtool/Attic/NewFileBrowser.st,v 1.2 2002-03-20 14:52:26 penk Exp $'
! !
